﻿<%@ Page Language="C#" AutoEventWireup="true" Inherits="Sitecore.sitecore.admin.AdminPage, Sitecore.Client"%>

<%@ Import Namespace="System.IO" %>
<%@ Import Namespace="System.Linq" %>
<%@ Import Namespace="System.Security.AccessControl" %>
<%@ Import Namespace="Sitecore" %>
<%@ Import Namespace="Sitecore.Configuration" %>
<%@ Import Namespace="Sitecore.Data" %>
<%@ Import Namespace="Sitecore.Data.Items" %>
<%@ Import Namespace="Sitecore.Data.Managers" %>

<script runat="server">
    private const string AppDataPath = "~/App_Data/";

    private static readonly ID ProfileDefinitionTempateId = new ID("{8E0C1738-3591-4C60-8151-54ABCC9807D1}");

    private readonly Tuple<ID, ID, Predicate<Item>>[] _oldDefinitionFolders =
    {
        new Tuple<ID, ID, Predicate<Item>>(ItemIDs.MediaLibraryRoot, null, Sitecore.Marketing.Definitions.MarketingAssets.Data.ItemDb.ItemExtensions.IsMarketingAsset), // Assets
        new Tuple<ID, ID, Predicate<Item>>(new ID("{EC095310-746F-4C1B-A73F-941863564DC2}"), new ID("{94FD1606-139E-46EE-86FF-BC5BF3C79804}"), null), // Campaigns
        new Tuple<ID, ID, Predicate<Item>>(new ID("{09CF9489-8F71-4A16-B707-4F013D8CDEFB}"), new ID("{F66716C7-19A0-4658-B2AC-AA7D180D5DF2}"), null), // Funnels
        new Tuple<ID, ID, Predicate<Item>>(new ID("{0CB97A9F-CAFB-42A0-8BE1-89AB9AE32BD9}"), new ID("{475E9026-333F-432D-A4DC-52E03B75CB6B}"), null), // Goals
        new Tuple<ID, ID, Predicate<Item>>(new ID("{062A1E69-0BF6-4D6D-AC4F-C11D0F7DC1E1}"), new ID("{EE43C2F0-6277-4144-B144-8CA2CEFCCF12}"), null), // Outcomes
        new Tuple<ID, ID, Predicate<Item>>(new ID("{12BD7E35-437B-449C-B931-23CFA12C03D8}"), ProfileDefinitionTempateId, null), // Profiles
        new Tuple<ID, ID, Predicate<Item>>(new ID("{633273C1-02A5-4EBC-9B82-BD1A7C684FEA}"), new ID("{059CFBDF-49FC-4F14-A4E5-B63E1E1AFB1E}"), null) // Page events
    };

    private readonly Tuple<ID, ID, Predicate<Item>>[] _newDefinitionFolders =
    {
        new Tuple<ID, ID, Predicate<Item>>(new ID("{7BF9F710-6CAD-4114-8DC7-88B4C4181562}"), new ID("{D4D43C33-3661-4630-AF63-0754CAB7B110}"), null), // Automation Plans
        new Tuple<ID, ID, Predicate<Item>>(new ID("{3C94F086-453B-48FC-9F1B-2B00BC0A55C7}"), new ID("{3F557952-B2BB-45EB-B625-E29698B2A165}"), null), // ContactList
        new Tuple<ID, ID, Predicate<Item>>(new ID("{A65E2D0F-27E1-49AD-B332-076745A74ED6}"), new ID("{A386C5F2-F7F1-4980-AEDE-72A5B84B7F45}"), null), // Events
        new Tuple<ID, ID, Predicate<Item>>(new ID("{CD304D58-D53B-420C-9D1A-C83255AF1554}"), new ID("{95D4ED35-1E47-48E9-95DB-DE0614A0E86C}"), null)  // Segments
    };

    private readonly ID[] _profileNestedDefinitionTemplateIds =
    {
        new ID("{0FC09EA4-8D87-4B0E-A5C9-8076AE863D9C}"), // Profile Card
        new ID("{4A6A7E36-2481-438F-A9BA-0453ECC638FA}"), // Pattern Card
        new ID("{44AB5107-3C73-42F0-A427-BEC549F944B9}"), // Profile Key 
    };

    private readonly Regex _aliasValidationRegex = new Regex(@"^[a-zA-Z0-9_\-\x20]*$");

    private Database ContextDb
    {
        get { return Factory.GetDatabase("master"); }
    }

    protected override void OnInit(EventArgs arguments)
    {
        CheckSecurity(true);
        Directory.CreateDirectory(Server.MapPath(AppDataPath));
        base.OnInit(arguments);
    }

    protected void Page_Load(object sender, EventArgs e)
    {
        if (!Page.IsPostBack)
        {
            FillRootsList();
        }
    }

    private void FillRootsList()
    {
        definitionRootsList.DataSource = _oldDefinitionFolders
            .Select(data => ContextDb.GetItem(data.Item1))
            .Where(item => item != null)
            .Select(item => item.Paths.ContentPath + " - " + item.ID)
            .OrderBy(s => s);
        definitionRootsList.DataBind();
    }

    private void ValidateAllDefinitions()
    {
        var allValidationErrors = new List<ItemError>();

        foreach (var definitionData in _oldDefinitionFolders)
        {
            allValidationErrors.AddRange(GetValidationErrors(definitionData.Item1, definitionData.Item2, definitionData.Item3));
        }


        using (var tw = File.CreateText(Server.MapPath(Path.Combine(AppDataPath, string.Format("DefinitionItemsNameValidationErrors_{0:yyyy_MM_dd_HH_mm_ss}.dat", DateTime.UtcNow)))))
        {
            tw.AutoFlush = true;
            foreach (var error in allValidationErrors)
            {
                tw.WriteLine("{0} - {1}: {2}", error.Category, error.IdsAsString, error.ErrorText);
            }
        }

        ErrorsList.DataSource = allValidationErrors;
        ErrorsList.DataBind();

        ResultPanel.Visible = true;
    }

    private IEnumerable<ItemError> GetValidationErrors(ID rootId, ID definitionTemplateId, Predicate<Item> includeFilter, string category = null)
    {
        var errors = new List<ItemError>();

        var rootItem = ContextDb.GetItem(rootId);
        if (rootItem != null)
        {
            var items = new List<Item>();
            LoadDefinitionItemsRecursively(rootItem, definitionTemplateId, items, includeFilter);

            category = category ?? rootItem.Name;
            errors.AddRange(GetNameUniquenessValidationErrors(category, items));
            errors.AddRange(GetNameValidationErrors(category, items));

            if (definitionTemplateId == ProfileDefinitionTempateId)
            {
                foreach (var profileItem in items)
                {
                    foreach (var profileNestedDefinitionTemplateId in _profileNestedDefinitionTemplateIds)
                    {
                        // All nested definitions of each profile should have valid name and unique alias.
                        errors.AddRange(GetValidationErrors(profileItem.ID, profileNestedDefinitionTemplateId, null, category));
                    }
                }
            }
        }

        return errors;
    }

    private IEnumerable<ItemError> GetNameValidationErrors(string category, List<Item> items)
    {
        foreach (var item in items)
        {
            if (!_aliasValidationRegex.IsMatch(item.Name))
            {
                yield return new ItemError
                {
                    Category = category,
                    Ids = new List<ID> { item.ID },
                    ErrorText = string.Format("Item name '{0}' consists not only of letters, digits, dashes or underscore.", item.Name)
                };
            }
        }
    }

    private IEnumerable<ItemError> GetNameUniquenessValidationErrors(string category, List<Item> items)
    {
        var duplicateGroups = items.GroupBy(x => x.Name.ToLower()).Where(g => g.Count() > 1).ToList();

        foreach (var duplicateGroup in duplicateGroups)
        {
            yield return new ItemError
            {
                Category = category,
                Ids = duplicateGroup.Select(g => g.ID).ToList(),
                ErrorText = string.Format("Multiple items have the same name - '{0}'.", duplicateGroup.First().Name)
            };
        }
    }

    private void LoadDefinitionItemsRecursively(Item parentItem, ID definitionTemplateId, List<Item> items, Predicate<Item> includeFilter)
    {
        if (parentItem.HasChildren)
        {
            foreach (Item child in parentItem.Children)
            {
                if (definitionTemplateId == (ID)null || TemplateManager.GetTemplate(child).InheritsFrom(definitionTemplateId))
                {
                    if (includeFilter == null || includeFilter(child))
                    {
                        items.Add(child);
                    }
                }

                LoadDefinitionItemsRecursively(child, definitionTemplateId, items, includeFilter);
            }
        }
    }

    private void validateItemNamesButton_OnClick(object sender, EventArgs e)
    {
        ValidateAllDefinitions();
    }

    private class ItemError
    {
        public string Category { get; set; }

        public string ErrorText { get; set; }

        public List<ID> Ids { get; set; }

        public string IdsAsString
        {
            get
            {
                if (Ids == null)
                {
                    return string.Empty;
                }

                return string.Join(", ", Ids);
            }
        }
    }
</script>

<!DOCTYPE html>
<html>
<head runat="server">
    <title>Sitecore Update Tool</title>
    <link rel="shortcut icon" href="/sitecore/images/favicon.ico" />
    <link href="/default.css" rel="stylesheet" />
    <style type="text/css">
        body {
            padding: 5px;
        }

        .note {
            background-color: lightgray;
        }

        .errortable {
            border: solid 1px grey;
            border-collapse: separate;
            border-spacing: 2px;
            margin-top: 10px;
            width: 100%;
        }

        .errortable tr:nth-child(even) { background: #CCC; }

        .errortable td { padding: 2px; }
    </style>
</head>

<body>
    <h1>Definition Items Name Validator</h1>
    <form runat="server" id="RootSelectionForm">
        <p class="note">Alias property was added to all marketing definitions in Sitecore 9.0. Alias is a unique key of definition within a given definition manager.
        Marketing Operations API requires that alias of marketing definition is unique and consists only of letters, digits, dashes, space or underscore.
        Name of marketing definition item is used as value of Alias property. This page validates names of marketing definition items so that names can be fixed before update package is installed.
        </p>
        <p>Definition items will be validated under the following root items:</p>
        <asp:BulletedList runat="server" ID="definitionRootsList" BorderWidth="0" />
        <asp:Button runat="server" ID="validateItemNamesButton" Text="Validate" OnClick="validateItemNamesButton_OnClick" />
    </form>

    <asp:Panel Visible="False" ID="ResultPanel" runat="server">
        <table class="errortable">
            <tr>
                <th>Category</th>
                <th>Ids</th>
                <th>Error Description</th>
            </tr>
            <asp:Repeater runat="server" ID="ErrorsList">
                <ItemTemplate>
                    <tr>
                        <td><%# Eval("Category") %></td>
                        <td><%# Eval("IdsAsString") %></td>
                        <td><%# Eval("ErrorText") %></td>
                    </tr>
                </ItemTemplate>
            </asp:Repeater>
        </table>
    </asp:Panel>

</body>
</html>
