GO
PRINT N'Start creating [xdb_collection].[UnlockContactIdentifiersIndex] stored procedure...';

GO
DROP PROCEDURE IF EXISTS [xdb_collection].[UnlockContactIdentifiersIndex]
GO
CREATE PROCEDURE [xdb_collection].[UnlockContactIdentifiersIndex]
    @BatchId UNIQUEIDENTIFIER -- The unique batch identifier.
    -- Assign execution context to OWNER of the module (stored procedure)
WITH EXECUTE AS OWNER 
AS
BEGIN
    -- SET NOCOUNT ON added to prevent extra result sets 
    -- (return a number of affected rows by the statements)
    -- in order to improve performance.
    SET NOCOUNT ON;
    SET XACT_ABORT ON;

    BEGIN TRY

        SET TRANSACTION ISOLATION LEVEL SNAPSHOT
        BEGIN TRANSACTION

        UPDATE [ContactIdentifiersIndex]
        SET
            [LockTime] = NULL -- LockTime == NULL means the contact identifier is inserted correctly before the lock time has expired.
        FROM
            [xdb_collection].[ContactIdentifiersIndex] AS [CII]
            INNER JOIN [xdb_collection].[UnlockContactIdentifiersIndex_Staging] AS [Staging]
                ON
                    DATALENGTH([Staging].[Identifier]) = [CII].[IdentifierHash]
                    AND [Staging].[Identifier] = [CII].[Identifier]
                    AND [Staging].[Source] = [CII].[Source] COLLATELatin1_General_BIN2

        WHERE
            [Staging].[BatchId] = @BatchId

        DELETE
        FROM 
            [xdb_collection].[UnlockContactIdentifiersIndex_Staging]
        WHERE
            [BatchId] = @BatchId;

        COMMIT TRANSACTION

    END TRY
    BEGIN CATCH

        IF @@TRANCOUNT > 0
            ROLLBACK TRANSACTION;

        ;THROW

    END CATCH

END

GO
PRINT N'Finish creating [xdb_collection].[UnlockContactIdentifiersIndex] stored procedure...';

GO
PRINT N'Start altering tables to remove default percentile constraints';

GO
IF  EXISTS (
	SELECT * 
	FROM sys.objects 
	WHERE name = N'ContactsPercentileDefault')

	ALTER TABLE [xdb_collection].[Contacts] 
	DROP CONSTRAINT [ContactsPercentileDefault]

GO
IF  EXISTS (
	SELECT * 
	FROM sys.objects 
	WHERE name = N'ContactsStagingPercentileDefault')

	ALTER TABLE [xdb_collection].[Contacts_Staging] 
	DROP CONSTRAINT [ContactsStagingPercentileDefault]

GO
IF  EXISTS (
	SELECT * 
	FROM sys.objects 
	WHERE name = N'InteractionsPercentileDefault')

	ALTER TABLE [xdb_collection].[Interactions] 
	DROP CONSTRAINT [InteractionsPercentileDefault]

GO
IF  EXISTS (
	SELECT * 
	FROM sys.objects 
	WHERE name = N'InteractionsStagingPercentileDefault')

	ALTER TABLE [xdb_collection].[Interactions_Staging] 
	DROP CONSTRAINT [InteractionsStagingPercentileDefault]

GO
PRINT N'Finish altering tables to remove default percentile constraints';